@@ -260,9 +260,8 @@ WPG2Parser::WPG2Parser(WPXInputStream *input, libwpg::WPGPaintInterface* painter
 	// default style
 	m_penForeColor = ::libwpg::WPGColor(0,0,0);
 	m_penBackColor = ::libwpg::WPGColor(0xff,0xff,0xff);
-	m_style.insert("svg:stroke-width", 0.001);
-//	m_pen.height = 0.0;
-	m_style.insert("libwpg:stroke-solid", true);
+	m_style.insert("svg:stroke-width", 0.0);
+	m_style.insert("draw:stroke", "solid");
 	m_dashArray = libwpg::WPGDashArray();
 	m_brushForeColor = ::libwpg::WPGColor(0,0,0);
 	m_brushBackColor = ::libwpg::WPGColor(0xff,0xff,0xff);
@@ -561,12 +560,12 @@ void WPG2Parser::handleStartWPG()
 		1,    18, 55,                            // style #6
 		3,    18, 55,  18, 55, 18, 127,          // style #7
 		2,   164, 55,  18, 55,                   // style #8
-		3,   145, 36, 138, 36, 18,  36,          // style #9
+		3,   145, 36,  18, 36, 18,  36,          // style #9
 		3,    91, 55,  91, 55, 18,  55,          // style #10
 		4,    91, 36,  91, 36, 18,  36, 18, 36,  // style #11
 		2,   182, 73,  73, 73,                   // style #12
 		3,   182, 36,  55, 36, 55,  36,          // style #13
-		3,   255, 73, 255, 73, 73,  73,          // style #14
+		3,   255, 73, 266, 73, 73,  73,          // style #14
 		4,   273, 36, 273, 36, 55,  36, 55, 36,  // style #15
 		0 // end marker
 	};
@@ -661,6 +660,8 @@ void WPG2Parser::flushCompoundPolygon()
 	
 	if (!context.compoundFilled)
 		tmpStyle.insert("draw:fill", "none");
+	if (!context.compoundFramed)
+		tmpStyle.insert("draw:stroke", "none");
 
 	if(context.compoundWindingRule)
 		tmpStyle.insert("svg:fill-rule", "nonzero");
@@ -798,7 +799,7 @@ void WPG2Parser::handlePenBackColor()
 		return;
 	if (!m_groupStack.empty() && m_groupStack.top().isCompoundPolygon())
 		return;
-#if 0
+
 	unsigned char red = readU8();
 	unsigned char green = readU8();
 	unsigned char blue = readU8();
@@ -806,12 +807,10 @@ void WPG2Parser::handlePenBackColor()
 
 	m_penBackColor = libwpg::WPGColor(red, green, blue, alpha);
 
-	m_dashArray.backColor = libwpg::WPGColor(red, green, blue, alpha);
 	m_style.insert("svg:stroke-color", m_penForeColor.getColorString());
 	m_style.insert("svg:stroke-opacity", m_penForeColor.getOpacity(), WPX_PERCENT);
 
 	WPG_DEBUG_MSG(("   Background color (RGBA): %d %d %d %d\n", red, green, blue, alpha));
-#endif
 }
 
 void WPG2Parser::handleDPPenBackColor()
@@ -820,7 +819,7 @@ void WPG2Parser::handleDPPenBackColor()
 		return;
 	if (!m_groupStack.empty() && m_groupStack.top().isCompoundPolygon())
 		return;
-#if 0
+
 	// we just ignore the least significant 8 bits
 	unsigned int red = (m_doublePrecision)   ? readU16()>>8 : readU8();
 	unsigned int green = (m_doublePrecision) ? readU16()>>8 : readU8();
@@ -829,12 +828,24 @@ void WPG2Parser::handleDPPenBackColor()
 
 	m_penBackColor = libwpg::WPGColor(red, green, blue, alpha);
 
-	m_dashArray.backColor = libwpg::WPGColor(red, green, blue, alpha);
 	m_style.insert("svg:stroke-color", m_penForeColor.getColorString());
 	m_style.insert("svg:stroke-opacity", m_penForeColor.getOpacity(), WPX_PERCENT);
 
 	WPG_DEBUG_MSG(("   Background color (RGBA): %d %d %d %d\n", red, green, blue, alpha));
-#endif
+}
+
+void WPG2Parser::setPenStyle()
+{
+	if (!m_style["draw:stroke"] || m_style["draw:stroke"]->getStr() != "dash")
+		return;
+	{
+		double strokeWidth = m_style["svg:stroke-width"] ? m_style["svg:stroke-width"]->getDouble() : 0.0;
+		m_style.insert("draw:dots1", m_dashArray.getDots1());
+		m_style.insert("draw:dots1-length", 72.0*72.0*strokeWidth*(m_dashArray.getDots1Length()), WPX_POINT);
+		m_style.insert("draw:dots2", m_dashArray.getDots2());
+		m_style.insert("draw:dots2-length", 72.0*72.0*strokeWidth*(m_dashArray.getDots2Length()), WPX_POINT);
+		m_style.insert("draw:distance", 72.0*72.0*strokeWidth*(m_dashArray.getDistance()), WPX_POINT);
+	}
 }
 
 void WPG2Parser::handlePenStyle()
@@ -852,11 +863,14 @@ void WPG2Parser::handlePenStyle()
 
 	m_dashArray = m_dashArrayStyles[style];
 	
-	if (!style)
+	if (!m_dashArray.getDots1() || !m_dashArray.getDots2())
 		m_style.insert("draw:stroke", "solid");
+	else
+		m_style.insert("draw:stroke", "dash");
+
+	setPenStyle();
 
 	WPG_DEBUG_MSG(("   Pen style : %d\n", style));
-	WPG_DEBUG_MSG(("   Segments : %d\n", m_dashArray.count()));
 }
 
 void WPG2Parser::handlePenSize()
@@ -950,8 +964,8 @@ void WPG2Parser::handleBrushGradient()
 	// TODO: get gradient extent
 
 	m_gradientAngle = angleInteger + (double)angleFraction/65536.0;
-	m_gradientRef.insert("svg:x", (double)xref);
-	m_gradientRef.insert("svg:y", (double)yref);
+	m_gradientRef.insert("svg:cx", (double)xref);
+	m_gradientRef.insert("svg:cy", (double)yref);
 
 	WPG_DEBUG_MSG(("       Gradient angle : %d.%d\n", angleInteger, angleFraction));
 	WPG_DEBUG_MSG(("   Gradient reference : %d.%d\n", xref, yref));
@@ -985,8 +999,8 @@ void WPG2Parser::handleDPBrushGradient()
 	// TODO: get gradient extent (in double precision)
 
 	m_gradientAngle = angleFraction + (double)angleInteger/65536.0;
-	m_gradientRef.insert("svg:x", (double)xref);
-	m_gradientRef.insert("svg:y", (double)yref);
+	m_gradientRef.insert("svg:cx", (double)xref);
+	m_gradientRef.insert("svg:cy", (double)yref);
 
 	WPG_DEBUG_MSG(("       Gradient angle : %d.%d\n", angleInteger, angleFraction));
 	WPG_DEBUG_MSG(("   Gradient reference : %d.%d\n", xref, yref));
@@ -1056,8 +1070,8 @@ void WPG2Parser::handleBrushForeColor()
 		// and they are actually in reverse order
 		if(count == 2)
 		{
-			double xref = m_gradientRef["svg:x"]->getDouble()/65536.0;
-			double yref = m_gradientRef["svg:y"]->getDouble()/65536.0;
+			double xref = m_gradientRef["svg:cx"]->getDouble()/65536.0;
+			double yref = m_gradientRef["svg:cy"]->getDouble()/65536.0;
 			double angle = m_gradientAngle*M_PI/180.0;
 			double tanangle = tan(angle);
 			double ref = (tanangle < 1e2 && tanangle > -1e2) ? (yref+xref*tanangle)/(1+tanangle) : xref;
@@ -1074,7 +1088,7 @@ void WPG2Parser::handleBrushForeColor()
 			propList.insert("svg:stop-opacity", colors[0].getOpacity(), WPX_PERCENT);
 			gradient.append(propList);
 			propList.clear();
-			if((m_gradientRef["svg:x"]->getInt() != 65535) && (m_gradientRef["svg:y"]->getInt() != 65535))
+			if((m_gradientRef["svg:cx"]->getInt() != 65535) && (m_gradientRef["svg:cy"]->getInt() != 65535))
 			{
 				propList.insert("svg:offset", 1.0, WPX_PERCENT);
 				propList.insert("svg:stop-color", colors[1].getColorString());
@@ -1149,8 +1163,8 @@ void WPG2Parser::handleDPBrushForeColor()
 		// and they are actually in reverse order
 		if(count == 2)
 		{
-			double xref = m_gradientRef["svg:x"]->getDouble()/65536.0;
-			double yref = m_gradientRef["svg:y"]->getDouble()/65536.0;
+			double xref = m_gradientRef["svg:cx"]->getDouble()/65536.0;
+			double yref = m_gradientRef["svg:cy"]->getDouble()/65536.0;
 			double angle = m_gradientAngle*M_PI/180.0;
 			double tanangle = tan(angle);
 			double ref = (tanangle<1e2) ? (yref+xref*tanangle)/(1+tanangle) : xref;
@@ -1168,7 +1182,7 @@ void WPG2Parser::handleDPBrushForeColor()
 			propList.insert("svg:stop-opacity", colors[0].getOpacity(), WPX_PERCENT);
 			gradient.append(propList);
 			propList.clear();
-			if((m_gradientRef["svg:x"]->getInt() != 65535) && (m_gradientRef["svg:y"]->getInt() != 65535))
+			if((m_gradientRef["svg:cx"]->getInt() != 65535) && (m_gradientRef["svg:cy"]->getInt() != 65535))
 			{
 				propList.insert("svg:offset", 1.0, WPX_PERCENT);
 				propList.insert("svg:stop-color", colors[1].getColorString());
@@ -1346,6 +1360,8 @@ void WPG2Parser::handlePolyline()
 	
 	if (!objCh.filled)
 		tmpStyle.insert("draw:fill", "none");
+	if (!objCh.framed)
+		tmpStyle.insert("draw:stroke", "none");
 
 	bool insideCompound = m_groupStack.empty() ? false :
 		m_groupStack.top().isCompoundPolygon();
@@ -1444,6 +1460,8 @@ void WPG2Parser::handlePolycurve()
 	
 	if (!objCh.filled)
 		tmpStyle.insert("draw:fill", "none");
+	if (!objCh.framed)
+		tmpStyle.insert("draw:stroke", "none");
 
 	bool insideCompound = m_groupStack.empty() ? false :
 		m_groupStack.top().isCompoundPolygon();
@@ -1521,6 +1539,8 @@ void WPG2Parser::handleRectangle()
 	
 	if (!objCh.filled)
 		tmpStyle.insert("draw:fill", "none");
+	if (!objCh.framed)
+		tmpStyle.insert("draw:stroke", "none");
 
 	long x1 = (m_doublePrecision) ? readS32() : readS16();
 	long y1 = (m_doublePrecision) ? readS32() : readS16();
@@ -1570,6 +1590,8 @@ void WPG2Parser::handleArc()
 	
 	if (!objCh.filled)
 		tmpStyle.insert("draw:fill", "none");
+	if (!objCh.framed)
+		tmpStyle.insert("draw:stroke", "none");
 
 	long cx = (m_doublePrecision) ? readS32() : readS16();
 	long cy = (m_doublePrecision) ? readS32() : readS16();
