@@ -28,21 +28,11 @@
 #define __LIBWPG_H__
 
 #define LIBWPG_VERSION_MAJOR     0
-#define LIBWPG_VERSION_MINOR     1
-#define LIBWPG_VERSION_REVISION  3
-#define LIBWPG_VERSION_STRING    "0.1.3"
+#define LIBWPG_VERSION_MINOR     2
+#define LIBWPG_VERSION_REVISION  1
+#define LIBWPG_VERSION_STRING    "0.2.1"
 
 #include "WPGraphics.h"
 #include "WPGPaintInterface.h"
 
-#include "WPGColor.h"
-#include "WPGPen.h"
-#include "WPGBrush.h"
-#include "WPGGradient.h"
-#include "WPGPath.h"
-#include "WPGPoint.h"
-#include "WPGRect.h"
-#include "WPGBitmap.h"
-#include "WPGBinaryData.h"
-
 #endif
